#ifndef COUNTDOWN_H
#define COUNTDOWN_H

#include <stdint.h>

// Alle 10 ms aufrufen wenn COUNTDOWN_US100 nicht define'd ist. 
// Alle 100 s aufrufen wenn COUNTDOWN_US100 define'd ist.
extern void job_countdown (void);

// In folgende Struktur die bentigten Countdown-Timer
// eintragen.
// Um ein Minute zu warten, ist es geschickter, einen 
// 10s-Zhler anzulegen und ihn auf 60 aufzuziehen anstastt
// einen 1m-Zhler anzulegen und ihn auf 1 aufzuziehen.
// Die Zhler werden auf 0 herabgezhlt und bleiben dann 
// stehen.
typedef struct
{
#ifdef COUNTDOWN_US100
    // Die Kompoinente muss mindestens das erste Elemenet enthalten
    struct
    {
        // Erstes Element ist fr internen Gebrauch!
        // Es bleibt nicht bei 0 stehen und wird immer wieder
        // von job_countdown aufgezogen, sobald der Zhler
        // abgelaufen ist.
		uint8_t timer_10ms; 
        // Ab hier die bentigten 8-Bit Countdown-Zhler eintragen
    } us100;
#endif /* COUNTDOWN_US100 */

    // Die Komponente muss mindestens das erste Elemenet enthalten
    struct
    {
        // Erstes Element ist fr internen Gebrauch!
        // Es bleibt nicht bei 0 stehen und wird immer wieder
        // von job_countdown aufgezogen, sobald der Zhler
        // abgelaufen ist.
		uint8_t timer_1s; 
        // Ab hier die bentigten 8-Bit Countdown-Zhler eintragen
		uint8_t wait_10ms;
		uint8_t led1;
		uint8_t led2;
		uint8_t spin;
		uint8_t debounceSW1_0;
		uint8_t debounceSW2_0;
		uint8_t debounceSW3_0;
		uint8_t debounceSW1_1s;
		uint8_t debounceSW2_1s;
		uint8_t debounceSW3_1s;
		uint8_t menu;
		uint8_t displayAtRun;
		uint8_t blink;
		uint8_t clock;
    } ms10;

    struct
    {
        // Ab hier die bentigten 16-Bit Countdown-Zhler eintragen
    } ms10_16;

    // Die Sekunden-Timer. 
    // Die Komponente darf auch komplett leer sein
    struct
    {
        // Erstes Element ist fr internen Gebrauch!
        // Es bleibt nicht bei 0 stehen und wird immer wieder
        // von job_countdown aufgezogen, sobald der Zhler
        // abgelaufen ist.
        uint8_t timer_1m;
        // Ab hier die bentigten 8-Bit Countdown-Zhler eintragen
		uint8_t menu;
		uint8_t debounceSW1_3s;
		uint8_t debounceSW2_3s;
		uint8_t debounceSW3_3s;
		uint8_t debounceSW1_5s;
		uint8_t debounceSW2_5s;
		uint8_t debounceSW3_5s;
		uint8_t displaySleepAfter;
		uint8_t offDelay;
    } sec;

    // Die Minuten-Timer. 
    // Die Komponente darf auch komplett leer sein
    struct
    {
        // Ab hier die bentigten 8-Bit Countdown-Zhler eintragen
    	//uint8_t test;
    } min;
} countdown_t;

extern volatile countdown_t count;

static inline
void wait_10ms (const uint8_t t)
{
#if defined (__GNUC__) && defined (__AVR__)
    asm volatile (" ; BARRIER" ::: "memory");
#endif // avr-gcc
    count.ms10.wait_10ms = 1+t;
    while (count.ms10.wait_10ms);
}

#endif /* COUNTDOWN_H */
