/*
 * prog.c
 *
 *  Created on: 15.04.2016
 *      Author: dkassen
 */

#include "menu.h"
#include <stdbool.h>
#include <stdint.h>
#include <util/delay.h>
#include "i2clcd.h"
#include "i2cmaster.h"
#include "countdown.h"

#include "display.h"
#include "properties.h"
#include "status.h"
#include "select.h"
#include "switches.h"
#include "adc.h"

#include <avr/eeprom.h>

void windUpClock()
{
	clockMin = durationM;
	clockSec_turn = turnTimeS;
	driveStatus = SPINUP;
	valuesChanged = true;
}

/*void eval_welome(bool swEnter, bool plus, bool minus)
{
	if (swEnter || plus || minus)
	{
		section = ProgOrStart;
		curserLine = 1;
	}
	print_Welcome();
}*/
bool submenu = false;
void menu_change(bool swEnter, bool plus, bool minus)
{
	print_editMenue();
	if (swEnter)
	{
		submenu = true;
		sectionChanged = true;
		return;
	}
	else if (plus)
	{
		if(section >= 3)
			section = 0;
		else
		{
			section++;
		}
		sectionChanged = true;
	}
	else if (minus)
	{
		if(section == 0)
			section = 3;
		else
		{
			section--;
		}
		sectionChanged = true;
	}
}

void menu_start(bool swEnter, bool plus, bool minus)
{
	if (swEnter)
	{
		windUpClock();
		section = RunRun;
		sectionChanged = true;
		return;
	}
	else if (plus || minus)
	{
		menu_change(swEnter, plus, minus);
		return;
	}
	else
		print_selectedStart();
}

void menu_changeDuration(bool swEnter, bool plus, bool minus)
{
	print_selectedDuration();
	if(submenu)
	{
		if (swEnter)
		{
			if(eeprom_read_byte(&NonVolatilerDURATION) != durationM)
			{
				eeprom_write_byte(&NonVolatilerDURATION, durationM );
			}
			sectionChanged = true;
			submenu = false;
			return;
		}
		else if (plus)
		{
			durationM = preventBetween(1, 100, 1, durationM);
			valuesChanged = true;
		}
		else if (minus)
		{
			durationM = preventBetween(1, 100, -1, durationM);
			valuesChanged = true;
		}
	}
	else
	{
		menu_change(swEnter, plus, minus);
	}
}

void menu_changeRPM(bool swEnter, bool plus, bool minus)
{
	print_selectRPM();
	if(submenu)
	{
		if(swEnter)
		{
			if(eeprom_read_word(&NonVolatilerRPMmax) != RPMmax)
			{
				eeprom_write_word(&NonVolatilerRPMmax, RPMmax );
			}
			sectionChanged = true;
			submenu = false;
			return;
		}
		else if(plus)
		{
			RPMmax = preventBetween(1600,2400,400,RPMmax);
			valuesChanged = true;
		}
		else if(minus)
		{
			RPMmax = preventBetween(1600,2400,-400,RPMmax);
			valuesChanged = true;
		}
	}
	else
	{
		menu_change(swEnter, plus, minus);
		return;
	}
}

void menu_changeTurn(bool swEnter, bool plus, bool minus)
{
	print_selectedTurnTime();
	if(submenu)
	{
		if (swEnter)
		{
			if(eeprom_read_byte(&NonVolatilerTURNTIME) != turnTimeS)
			{
				eeprom_write_byte(&NonVolatilerTURNTIME, turnTimeS );
			}
			sectionChanged = true;
			submenu = false;
			return;
		}
		else if (plus)
		{
			turnTimeS = preventBetween(30, 180, 30, turnTimeS);
			valuesChanged = true;
		}
		else if (minus)
		{
			turnTimeS = preventBetween(30, 180, -30, turnTimeS);
			valuesChanged = true;
		}
	}
	else
	{
		menu_change(swEnter, plus, minus);
	}
}

void menu_changeInputPower(bool swEnter, bool plus, bool minus)
{
	print_selectedVDC();
	//Totaler quatsch, da wir max in EINE Richtung den Output statt den Input messen
	//Hier brauchen wir im nchsten Design einen ADC Kanal mit Spannungsteiler an den EINGANG!
	uint16_t adcval = ADC_Read_Avg(2, 4);  // Kanal 2, Mittelwert aus 4 Messungen
	if (swEnter)
	{
		if(eeprom_read_byte(&NonVolatilerVDC) != VDC)
		{
			eeprom_write_byte(&NonVolatilerVDC, VDC );
		}
		section = ProgOrStart;
		sectionChanged = true;
		return;
	}
	else if (plus)
	{
		VDC = preventBetween(15,24,1,VDC);
	}
	else if (minus)
	{
		VDC = preventBetween(15,24,-1,VDC);
	}

}
void menu_changeScreenSaver(bool swEnter, bool plus, bool minus)
{
	print_selectedBackLight();
	if (swEnter)
	{
		if(eeprom_read_byte(&NonVolatilerBacklightSec) != BacklightSec)
		{
			eeprom_write_byte(&NonVolatilerBacklightSec, BacklightSec );
		}
		section = ProgOrStart;
		sectionChanged = true;
		return;
	}
	else if (plus)
	{
		BacklightSec = preventBetween(0,200,5,BacklightSec);
	}
	else if (minus)
	{
		BacklightSec = preventBetween(0,200,-5,BacklightSec);
	}

}

void runningClock()
{
	if(count.ms10.clock == 0)
	{
		if(clockSec > 0)
			clockSec--;
		else
		{
			clockSec = 59;
			if(clockMin > 0)
				clockMin--;
		}

		if(clockSec_turn > 0)
			clockSec_turn--;
		else
		{
			clockSec_turn = turnTimeS-1;
			readyForDriveToggle = true;
		}
		count.ms10.clock = 100;
	}
}

void menu_run(bool swEnter, bool plus, bool minus)
{
	runningClock();
	print_runningCleaner();
	if (swEnter || (clockMin == 0 && clockSec == 0))
	{
		submenu = false;
		section = ProgOrStart;
		sectionChanged = true;
	}
	if(swEnter)
		durationM = clockMin;
}

enum MenueSection section = ProgOrStart;

void menu_update()
{
	if(sw1_0sec || sw2_0sec || sw3_0sec || sw1_1sec || sw2_1sec || sw3_1sec || sw1_3sec || sw2_3sec || sw3_3sec || sw1_5sec || sw2_5sec || sw3_5sec)
	{
		displayWakeUp();
		//freeSwitches();
	}
	if(sw1_3sec && sw2_3sec && sw3_3sec)
	{
		section = ProgPower;
	}
	if(sw3_5sec)
	{
		section = ProgBacklight;
	}
	bool swEnter = sw1_0sec;
	bool plus    = sw2_0sec;
	bool minus   = sw3_0sec;
	freeSwitches();
	//showSwitches();

	switch (section)
	{
	//case Welcome:
		//eval_welome(swEnter, plus, minus);
		//break;
	case ProgOrStart:
		menu_start(swEnter, plus, minus);
		break;
	case ProgDuration:
		menu_changeDuration(swEnter, plus, minus);
		break;
	case ProgRPM:
		menu_changeRPM(swEnter, plus, minus);
		break;
	case ProgTurn:
		menu_changeTurn(swEnter, plus, minus);
		break;
	case ProgPower:
		menu_changeInputPower(swEnter, plus, minus);
		break;
	case ProgBacklight:
		menu_changeScreenSaver(swEnter, plus, minus);
		break;
	case RunAbbort:
	case RunRun:
		menu_run(swEnter, plus, minus);
		break;
	}
}

void showSwitches()
{
	lcd_gotolc(1, 14);
	if (sw1_0sec)
		lcd_print_P(PSTR("1"));
	if (sw2_0sec)
		lcd_print_P(PSTR("2"));
	if (sw3_0sec)
		lcd_print_P(PSTR("3"));
	lcd_gotolc(2, 14);
	if (sw1_1sec)
		lcd_print_P(PSTR("1"));
	if (sw2_1sec)
		lcd_print_P(PSTR("2"));
	if (sw3_1sec)
		lcd_print_P(PSTR("3"));
//		else if (sw1_5sec)
//			lcd_printlc_P(1, 1, PSTR("Taster 15pressed"));
//		else if (sw2_5sec)
//			lcd_printlc_P(1, 1, PSTR("Taster 25pressed"));
//		else if (sw3_5sec)
//			lcd_printlc_P(1, 1, PSTR("Taster 35pressed"));
//		else
//			lcd_printlc_P(1, 1, PSTR("   NON pressed  "));
}

