#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>

#include "timer2.h"
#include "countdown.h"

/* Dieses Modul kmmert sich um die Initialisierung von
 * Timer2, damit er IRQS Interrupts pro Sekunde erzeugt.
 * Dies dient dann als Basis fr den 10ms-Takt, in dem
 * job_countdown() ausgefhrt wird um die Countdown-Zhler
 * zu aktualisieren.
 * Wird im folgenden keine Warnung und kein Fehler erzeugt, 
 * dann arbeitet Timer2 im Rahmen der Genauigkeit der 
 * zugrundeliegenden Taktquelle *exakt* mit der angeforderten 
 * IRQ-Rate.
 */
 
#if !defined (__AVR_ATmega88__)     \
    && !defined (__AVR_ATmega88A__)
	&& !defined (__AVR_ATmega168__)
#   error Dieses Beispielprogramm ist geschrieben fuer 
#   error einen AVR ATmaga88/168. Wenn du einen anderen C
#   error verwensest, dann fge entsprechenden Code einfach
#   error unten hinzu.
#endif /* ATmega88 */

// Vorteiler fr den Timer
#define PRESCALE 8

// Testet Plausibilitt der Einstellungen fr
// F_CPU, PRESCALE und IRQS

#if F_CPU / PRESCALE / IRQS -1 < 1 \
    || F_CPU / PRESCALE / IRQS -1 > 0xff
#   error Timer2 schafft das nicht! 
#   error Nimm andere Werte fuer F_CPU, PRESCALE, IRQS.
#endif

#if (F_CPU % (PRESCALE * IRQS)) != 0
#   warning Timer2 arbeitet ungenau
#endif

void timer2_init (void)
{
    // Mode #2 (CTC) und PRESCALE=8
    TCCR2A = (1 << WGM21);
    TCCR2B = (1 << CS21);

    // OutputCompare fr gewnschte Timer2 Frequenz
    OCR2A = (uint16_t) ((uint32_t) F_CPU / PRESCALE / IRQS -1);

    // OutputCompare-Interrupt A fr Timer 2
    TIMSK2 |= (1 << OCIE2A);
}

////////////////////////////////////////////////////////////////
#if (IRQS % 100) != 0
#   warning Mit diesem Wert fuer IRQS werden nicht genau 10ms erzeugt.
#   warning Nimm ein Vielfaches von 100.
#endif

SIGNAL (TIMER2_COMPA_vect)
{
    static uint16_t irqs_10ms;
    
    wdt_reset();
    
    ////////////////////////////////////////////////////////////
    // 10 ms-Takt fr Jobs: countdown-Zhler, Tasten-Entprellung, DCF, ...
    irqs_10ms = 1 + irqs_10ms;

    // Sind 10 ms voll?
    // IRQS wird im Makefile definiert, es gibt die Anzahl der 
    // IRQs pro Sekunde an.
    // Um 10ms zu erhalten, teilen wir also durch 100
    if (irqs_10ms >= IRQS / 100)
        irqs_10ms = 0;

    // Fhre nicht alle Jobs gleichzeitig aus, damit eine ISR nicht
    // zu lange dauert. Stattdessen staffeln wir die Aufrufe: es gibt
    // nur einen je 10ms, aber immer einen anderen (oder garkeinen).
    
    if (0 == irqs_10ms)   job_countdown();
    // if (1 == irqs_10ms)   mach_was_alle_10ms();
    // if (2 == irqs_10ms)   mach_nochwas_alle_10ms();
}
